@@ -252,7 +252,7 @@ package Perlito5::AST::For;
         my $cond;
         if (ref($self->{cond}) eq 'ARRAY') {
             # C-style for
-            $cond = [ map { defined ? $_->emit_compile_time() : $_ } @{$self->{cond}} ];
+            $cond = [ map { defined($_) ? $_->emit_compile_time() : $_ } @{$self->{cond}} ];
         }
         else {
             $cond = $self->{cond}->emit_compile_time(),
