@@ -108,7 +108,7 @@ sub term_special_var {
     #  ' ${main::x} '
     #
 
-    my $s = substr( $str, $pos, 3 );
+    my $s = $str->[$pos] . $str->[$pos + 1] . $str->[$pos + 2];
     if ( $s eq '$#[' ) {
         # special case: $# is not valid, but @# is ok
         $len = 2;
@@ -117,21 +117,19 @@ sub term_special_var {
         $len = length($s);
     }
     else {
-        $s = substr( $str, $pos, 2 );
+        $s = $str->[$pos] . $str->[$pos + 1];
         if ( exists $special_var{$s} ) {
             $len = 2;
         }
     }
     if ( $len ) {
-        my $c0 = substr( $str, $pos + $len - 1, 1 );
-        my $c1 = substr( $str, $pos + $len, 1 );
+        my $c0 = $str->[ $pos + $len - 1 ];
+        my $c1 = $str->[ $pos + $len ];
         if  ( 
-                ( $c0 eq '$' || $c0 eq '@' || $c0 eq '%' || $c0 eq '*' || $c0 eq '&' )
+                ( $Perlito5::Grammar::is_var_sigil{$c0} )
             &&  
-                ( $c1 eq '$' || $c1 eq '@' || $c1 eq '%' || $c1 eq '*' || $c1 eq '&' 
-                || ( $c1 ge 'a' && $c1 le 'z' )
-                || ( $c1 ge 'A' && $c1 le 'Z' )
-                || ( $c1 ge '0' && $c1 le '9' )
+                ( $Perlito5::Grammar::is_var_sigil{$c1}
+                || $Perlito5::Grammar::is_ident_middle{$c1}
                 )
             ) 
         {
@@ -169,12 +167,12 @@ sub term_sigil {
     my $str = $_[0];
     my $pos = $_[1];
 
-    my $c1 = substr($str, $pos, 1);
+    my $c1 = $str->[$pos];
     return unless exists $sigil{$c1};
 
     my $p = $pos + 1;
     my $sigil = $c1;
-    if (substr($str, $pos, 2) eq '$#') {
+    if ( $c1 eq '$' && $str->[$pos+1] eq '#') {
         $sigil = '$#';
         $p++;
     }
@@ -183,7 +181,7 @@ sub term_sigil {
     $p = $m->{to};
 
     my $p0 = $p;
-    $c1 = substr($str, $p, 1);
+    $c1 = $str->[$p];
     my $q = $p + 1;
     if ( $c1 eq '{' ) {
         #  ${ ...
@@ -202,11 +200,19 @@ sub term_sigil {
                 $name = Perlito5::Match::flat($n);
                 $pos  = $n->{to};
             }
-            if ($namespace || $name) {
+            if (
+                ( $namespace || $name )
+                && !(
+                       ( $namespace eq 'CORE' || $namespace eq '' )
+                    && ( $name eq 'sub' )
+                )
+              )
+            {
                 my $spc = Perlito5::Grammar::Space::opt_ws($str, $pos);
-                if (substr($str, $pos, 1) eq '{' || substr($str, $pos, 1) eq '[' || substr($str, $pos, 1) eq '}') {
+                my $pos = $spc->{to};
+                if ($str->[$pos] eq '{' || $str->[$pos] eq '[' || $str->[$pos] eq '}') {
                     # we are not parsing:  ${subr()}
-                    # we are parsing:  ${var}  ${var{index}}
+                    # we are parsing:  ${var}  ${var{index}}  ${var[index]}
                     # create the 'Var' object
                     $m->{capture} = Perlito5::AST::Var->new(
                         sigil       => $sigil,
@@ -219,11 +225,34 @@ sub term_sigil {
                     $m->{capture} = [ 'term', $m->{capture} ];
                     $spc = Perlito5::Grammar::Space::opt_ws($str, $m->{to});
                     my $p = $spc->{to};
-                    if ( substr($str, $p, 1) eq '}' ) {
+                    if ( $str->[$p] eq '}' ) {
                         $m->{to} = $p + 1;
                         return $m;
                     }
                 }
+                elsif ($str->[$pos] eq '-' && $str->[$pos + 1] eq '>') {
+                    my $spc = Perlito5::Grammar::Space::opt_ws($str, $pos + 2);
+                    my $pos = $spc->{to};
+                    if ($str->[$pos] eq '{' || $str->[$pos] eq '[') {
+                        # we are parsing:  ${var->{index}}  ${var->[index]}
+                        # create the 'Var' object
+                        $m->{capture} = Perlito5::AST::Var->new(
+                            sigil       => $sigil,
+                            namespace   => ( $namespace || $Perlito5::PKG_NAME ),  # deref variable is always global
+                            name        => $name,
+                        );
+                        $m->{to} = $spc->{to};
+                        # hijack some string interpolation code to parse the subscript
+                        $m = Perlito5::Grammar::String::double_quoted_var_with_subscript($m);
+                        $m->{capture} = [ 'term', $m->{capture} ];
+                        $spc = Perlito5::Grammar::Space::opt_ws($str, $m->{to});
+                        my $p = $spc->{to};
+                        if ( $str->[$p] eq '}' ) {
+                            $m->{to} = $p + 1;
+                            return $m;
+                        }
+                    }
+                }
             }
         }
         my $caret = Perlito5::Grammar::caret_char( $str, $p );
@@ -236,7 +265,7 @@ sub term_sigil {
                 $name = $name . Perlito5::Match::flat($m);
                 $p = $m->{to};
             }
-            if ( substr($str, $p, 1) eq '}' ) {
+            if ( $str->[$p] eq '}' ) {
                 $caret->{capture} = [ 'term', 
                         Perlito5::AST::Var->new(
                             name => $name,
@@ -248,19 +277,19 @@ sub term_sigil {
                 return $caret;
             }
         }
-        my $special = $sigil . substr($str, $p, 1);
+        my $special = $sigil . $str->[$p];
         if ( exists $special_var{$special} ) {
             # ${@}  $#{+}  - special variable
             my $m = Perlito5::Grammar::Space::opt_ws($str, $p + 1);
             my $p2 = $m->{to};
-            my $c2 = substr($str, $p2, 1);
+            my $c2 = $str->[$p2];
             if ($c2 eq '}') {
                 $m->{to} = $p2 + 1;
                 $m->{capture} = [ 'term', 
                         Perlito5::AST::Var->new(
                                 sigil       => $sigil,
                                 namespace   => '',
-                                name        => substr($str, $p, 1),
+                                name        => $str->[$p],
                                 ( $sigil eq '$#' ? ( _real_sigil => '@' ) : () ),
                                 _namespace  => 'main',
                             )
@@ -268,9 +297,9 @@ sub term_sigil {
                 return $m;
             }
         }
-        if (substr($str, $p, 1) eq '}') {
+        if ($str->[$p] eq '}') {
             # ${}
-            die "syntax error";
+            Perlito5::Compiler::error( "syntax error" );
         }
         $m = Perlito5::Grammar::block( $str, $p0 );
         if ($m) {
@@ -281,6 +310,7 @@ sub term_sigil {
                && (  ref($ast->{stmts}[0]) eq 'Perlito5::AST::Apply'
                   || ref($ast->{stmts}[0]) eq 'Perlito5::AST::Call' 
                   || ref($ast->{stmts}[0]) eq 'Perlito5::AST::Var' 
+                  || ref($ast->{stmts}[0]) eq 'Perlito5::AST::Buf' 
                   || ref($ast->{stmts}[0]) eq 'Perlito5::AST::Index' 
                   || ref($ast->{stmts}[0]) eq 'Perlito5::AST::Lookup' )
             ) {
@@ -290,6 +320,7 @@ sub term_sigil {
                         code      => 'prefix:<' . $sigil . '>',
                         namespace => '',
                         arguments => [ $ast->{stmts}[0] ],
+                        _strict_refs => ( $^H & $Perlito5::STRICT_REFS ),
                     ),
                 ];
                 return $m;
@@ -305,7 +336,8 @@ sub term_sigil {
                             namespace => '',
                             arguments => [ $ast ],
                         )
-                    ]
+                    ],
+                    _strict_refs => ( $^H & $Perlito5::STRICT_REFS ),
                 )
             ];
             return $m;
@@ -326,21 +358,24 @@ sub term_sigil {
     }
     if ( $c1 eq '$' ) {
         #  $$ ...
-        my $m2 = Perlito5::Grammar::Space::opt_ws($str, $p + 1);
-        my $p2 = $m2->{to};
-        my $c2 = substr($str, $p2, 1);
-        if ($c2 ne ',' && $c2 ne ';') {
-            # not $$; not $$,
-            $m = term_sigil( $str, $p );
-            if ($m) {
-                $m->{capture} = [ 'term',  
-                        Perlito5::AST::Apply->new( 
-                                'arguments' => [ $m->{capture}[1] ],
-                                'code'      => 'prefix:<' . $sigil . '>', 
-                                'namespace' => ''
-                            )
-                    ];
-                return $m;
+        my $m2 = Perlito5::Grammar::Space::ws($str, $p + 1);
+        if (!$m2) {
+            my $p2 = $p + 1;
+            my $c2 = $str->[$p2];
+            if ( $c2 eq '_' || $c2 eq '$' || !exists $special_var{ '$' . $c2 } ) {
+                # not '$$;' not '$$,' not '$$+1'
+                $m = term_sigil( $str, $p );
+                if ($m) {
+                    $m->{capture} = [ 'term',  
+                            Perlito5::AST::Apply->new( 
+                                    arguments  => [ $m->{capture}[1] ],
+                                    code       => 'prefix:<' . $sigil . '>', 
+                                    namespace  => '',
+                                    _strict_refs => ( $^H & $Perlito5::STRICT_REFS ),
+                                )
+                        ];
+                    return $m;
+                }
             }
         }
     }
@@ -404,7 +439,7 @@ This module parses source code for Perl 5 statements and generates Perlito5 AST.
 =head1 AUTHORS
 
 Flavio Soibelmann Glock <fglock@gmail.com>.
-The Pugs Team E<lt>perl6-compiler@perl.orgE<gt>.
+The Pugs Team.
 
 =head1 COPYRIGHT
 
