@@ -148,11 +148,11 @@ package Perlito5::Java::LexicalBlock;
             else {
                 if (!@{$last_statement->{arguments}}) {
                     $Perlito5::THROW_RETURN = 1;
-                    push @str, 'return PerlOp.ret(PerlOp.context(return_context));'; 
+                    push @str, 'throw new PlReturnException(PerlOp.context(return_context));'; 
                 }
                 else {
                     $Perlito5::THROW_RETURN = 1;
-                    push @str, 'return PerlOp.ret('
+                    push @str, 'throw new PlReturnException('
                         . Perlito5::Java::to_runtime_context([$last_statement->{arguments}[0]], $level+1, 'return')
                         . ');';
                 }
@@ -1209,15 +1209,15 @@ package Perlito5::AST::Var;
             return $sub . '.apply(' . Perlito5::Java::to_context($wantarray) . ', List__)';
         }
         if ($sigil eq '@') {
-            if ($self->{sigil} eq '$#') {
-                return "PlV.array_get$local(" . $index . ').end_of_array_index()'
-            }
             my $s = "PlV.array_get$local(" . $index . ')';
             if (!$local) {
                 # create a PlStringConstant
                 my $scalar = Perlito5::AST::Buf->new( buf => $full_name )->emit_java($level, 'scalar');
                 $s = $scalar . '.arrayRef.o.array_deref_strict()';
             }
+            if ($self->{sigil} eq '$#') {
+                return $s . '.end_of_array_index()'
+            }
             if ( $wantarray eq 'scalar' ) {
                 return $s . '.length_of_array()';
             }
@@ -1631,7 +1631,7 @@ package Perlito5::AST::Call;
             if (  ref( $self->{invocant} ) eq 'Perlito5::AST::Var' && $self->{invocant}{sigil} eq '::' ) {
                 if ( $self->{invocant}{namespace} eq '__SUB__' || $self->{invocant}{namespace} eq 'CORE::__SUB__' ) {
                     # __SUB__->()
-                    $invocant = 'this.getCurrentSub()';     # "this" is the closure
+                    $invocant = 'this.currentSub';     # "this" is the closure
                 }
                 else {
                     # x->()
@@ -2149,7 +2149,7 @@ package Perlito5::AST::Sub;
         }
 
         my $outer_sub;
-        $outer_sub = 'this.getCurrentSub()' if $Perlito5::Java::is_inside_subroutine;
+        $outer_sub = 'this.currentSub' if $Perlito5::Java::is_inside_subroutine;
 
         my $sub_ref = Perlito5::Java::get_label();
         local $Perlito5::Java::is_inside_subroutine = 1;
