@@ -114,6 +114,13 @@ package Perlito5::AST::Apply;
     sub emit_javascript2_set {
         my ($self, $arguments, $level, $wantarray) = @_;
         my $code = $self->{code};
+        if ($code eq 'vec') {
+            return '(' . $self->{arguments}->[0]->emit_javascript2( $level, 'scalar' ) . ' = '
+                .       'p5vec_set('
+                .           join( ', ', map( $_->emit_javascript2, (@{ $self->{arguments} }, $arguments) ) )
+                .       ')'
+                . ')';
+        }
         if ($code eq 'prefix:<$>') {
             return 'p5scalar_deref_set(' 
                 . Perlito5::JavaScript2::emit_javascript2_autovivify( $self->{arguments}->[0], $level+1, 'scalar' ) . ', '
@@ -555,7 +562,21 @@ package Perlito5::AST::Apply;
                 my $key_js = $arg->autoquote($arg->{index_exp})->emit_javascript2($level);
                 my $suffix = ((  $v->isa('Perlito5::AST::Var') && $v->{sigil} eq '$') ? '' : '._hash_');
 
-                return "((function (v,k) { var ret = v[k]; delete (v[k]); return ret;})(" . $v_js . $suffix . ',' . $key_js . "))";
+                if (!defined($v->{name})) {
+                    # delete $Module::{foo}
+                    # delete @Module::{qw/foo bar/}
+                    my $index = Perlito5::JavaScript2::escape_string($v->{namespace});
+                    return "p5deleteSymbolTable(" . $index . ', ' . $arg->autoquote($arg->{index_exp})->emit_javascript2($level) . ')';
+                }
+
+                if (  (ref($v) eq 'Perlito5::AST::Var')
+                   && $v->{sigil} eq '@'
+                   )
+                {
+                    return "p5hash_delete_list(" . $v_js . $suffix . ',' . $key_js . ")";
+                }
+
+                return "p5hash_delete(" . $v_js . $suffix . ',' . $key_js . ")";
             }
             if ($arg->isa( 'Perlito5::AST::Index' )) {
                 my $v = $arg->obj;
