@@ -1138,8 +1138,6 @@ p5sort = function(namespace, func, args) {
 perl5_to_js = function( source, namespace, var_env_js, p5want ) {
     // CORE.say(["source: [" + source + "]"]);
 
-    var strict_old = p5global("$", "Perlito5", "STRICT").FETCH();
-
     var namespace_old = p5global("$", "Perlito5", "PKG_NAME").FETCH();
     p5pkg["Perlito5"].v_PKG_NAME.assign(namespace);
 
@@ -1167,7 +1165,6 @@ perl5_to_js = function( source, namespace, var_env_js, p5want ) {
     // CORE.say(["js-source: [" + js_code + "]"]);
 
     p5pkg["Perlito5"].v_PKG_NAME.assign(namespace_old);
-    p5pkg["Perlito5"].v_STRICT.assign(strict_old);
     return js_code;
 }
 
